@@ -38,6 +38,10 @@ impl TcpListenProcessor {
 impl Processor for TcpListenProcessor {
     type Context = Context;
 
+    async fn initialize(&mut self, ctx: &mut Context) -> Result<()> {
+        ctx.set_cluster(crate::CLUSTER_NAME).await
+    }
+
     async fn process(&mut self, ctx: &mut Self::Context) -> Result<bool> {
         trace!("Waiting for incoming TCP connection...");
 
